package net.tonreg.emf.edapt.example.ui.parts;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.edapt.migration.MigrationException;
import org.eclipse.emf.edapt.migration.ReleaseUtils;
import org.eclipse.emf.edapt.migration.execution.Migrator;
import org.eclipse.emf.edapt.migration.execution.MigratorRegistry;
import org.eclipse.emf.edapt.spi.history.Release;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

public class ContentPart {

	private static final String PLUGIN_ID = "net.tonreg.emf.edapt.example.ui";
	private static final String FILE_PATH = "resources/data.extension";

	private TreeViewer treeViewer;

	private AdapterFactory adapterFactory;

	public ContentPart() {
		adapterFactory = new ComposedAdapterFactory(ComposedAdapterFactory.Descriptor.Registry.INSTANCE);

	}

	@PostConstruct
	public void createComposite(Composite parent) {
		parent.setLayout( new GridLayout(1, false) );

		treeViewer = new TreeViewer(parent);
		treeViewer.getTree().setLayoutData( new GridData(GridData.FILL_BOTH) );

		treeViewer.setContentProvider( new AdapterFactoryContentProvider(adapterFactory) );
		treeViewer.setLabelProvider( new AdapterFactoryLabelProvider(adapterFactory) );

		treeViewer.setInput(getData());
		treeViewer.expandAll();
	}

	@Focus
	public void setFocus() {
		treeViewer.getTree().setFocus();
	}

	private Object getData() {
		URI resourceURI = URI.createPlatformPluginURI(PLUGIN_ID + "/" + FILE_PATH, false);

		// Check if migration is needed
		checkMigration(resourceURI);

		// Get resource
		final ResourceSet resourceSet = new ResourceSetImpl();
//		Resource resource = resourceSet.createResource(resourceURI);
		Resource resource = resourceSet.getResource(resourceURI, true);

		// Load resource
		Map<Object, Object> loadOptions = resourceSet.getLoadOptions();
		try {
			resource.load(loadOptions);
			resourceSet.getResources().add(resource);
		} catch (IOException e) {
			e.printStackTrace();
		}

		return resource;
	}

	private void checkMigration(final URI resourceURI) {
		String nsURI = ReleaseUtils.getNamespaceURI(resourceURI);

		final Migrator migrator = MigratorRegistry.getInstance().getMigrator(nsURI);

		if (migrator != null) {
			final Release release = migrator.getRelease(resourceURI).iterator().next();

			if (!release.isLatestRelease()) {
				if (MessageDialog.openQuestion(Display.getDefault().getActiveShell(),
						"Migration necessary",
						"A migration of the model is necessary. " +
						"Do you want to proceed?")) {

					performMigration(migrator, resourceURI, release);
				}
			}
		} else {
			System.out.println("No migrator found.");
		}
	}

	private void performMigration(final Migrator migrator, final URI resourceURI, final Release release) {

		IRunnableWithProgress runnable = new IRunnableWithProgress() {

			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
//					ResourceSet resourceSet = 
					migrator.migrateAndLoad(Collections.singletonList(resourceURI), release, null, monitor);
//					editingDomain.getResourceSet().getResources().addAll(resourceSet.getResources());
				} catch (MigrationException e) {
					throw new InvocationTargetException(e);
				}
			}

		};

		try {
			new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run(false, false, runnable);
		} catch (InvocationTargetException e) {
			MessageDialog.openError(
					Display.getDefault().getActiveShell(),
					"Migration error",
					"An error occured during migration of the model. " + 
					"More information on the error can be found in the error log.");
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

}
